#!/usr/bin/env python3

import re
import os.path
import sys

#VERSION  October 14, 2022

# Replace empty modules in source file with module contents from module library.
# This script works much like the module program from the Delila package.

#Synopsis: replace.py rawfile modfile outfile 

# --------------------------------------------------------
# Skip lines in module file until you come to the next module
# or end of file.
def FindNextModule(F,line) :
    while not line.startswith(BeginMod) and line != '' :
        line = F.readline()
    return line

# --------------------------------------------------------
# Add lines to the module until the end of module is reached
def MakeModule(line,ModCount) :
    tokens = line.split(" ")
    name = tokens[3]
    print("============= found module: " + name)
    mods[name] = []
    while not line.startswith(EndMod) :
        mods[name].append(line)
        print(line)
        line = modfile.readline()
    #mods[name].append(line) # don't forget to append the END MODULE line
    #print(line)
    line = modfile.readline()
    ModCount = ModCount + 1
    return line, ModCount

# --------------------------------------------------------
# Write the lines of a module to the output file
def WriteMods(outfile,name) :
    for line in mods[name] :
        outfile.write(line)
    return

# --------------------------------------------------------
# Copy lines from rawfile to outfile until you come to the next module
# or end of file.
def CopyLines(rawfile,outfile,line) :
    while not line.startswith(BeginMod) and line != '' :
        outfile.write(line)
        line = rawfile.readline()
    return line

# --------------------------------------------------------
# Add lines to the module until the end of module is reached
def ReplaceModule(rawfile,line,outfile,ModRep) :
    #outfile.write(line)
    tokens = line.split(" ")
    name = tokens[3]
    print("============= replacing module: " + name)
    WriteMods(outfile,name)
    line = rawfile.readline()
    #outfile.write(line)
    ModRep = ModRep + 1
    return line, ModCount

#=========================  MAIN ============================
# ensure that there are enough command line arguments to parse 
if len(sys.argv) < 3:
    print("Usage: replacemods.py rawfile modfile outfile")
    sys.exit();

#Convert arguments to variables
RAWFN    = sys.argv[1]
MODFN    = sys.argv[2]
OUTFN   = sys.argv[3]


if not os.path.exists(RAWFN) :
    print(RAWFN + " not found")
elif not os.path.exists(MODFN) :
    print(MODFN + " not found")
else :
    BeginMod = "(* BEGIN MODULE" # beginning of a module
    EndMod= "(* END MODULE" # end of a module
    ModCount = 0

    # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    # Read module file and create a dictionary of modules.
    modfile = open(MODFN,'r')
    mods = {}
    nextline = modfile.readline()
    while nextline != '' :
        nextline = FindNextModule(modfile,nextline)
        if nextline != '' :
            nextline,ModCount = MakeModule(nextline,ModCount)
            print(nextline)
    print(MODFN + ": " + str(ModCount) + " modules found")
    for key in mods :
        print(key)
    modfile.close()

    # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    # Read the raw file and write lines to outfile, replacing
    # module references with the code of each module.
    rawfile = open(RAWFN,'r')
    outfile = open(OUTFN,'w')
    nextline = rawfile.readline()
    ModRep = 0
    while nextline != '' :
        nextline = CopyLines(rawfile,outfile,nextline)
        if nextline != '' :
            nextline, ModRep = ReplaceModule(rawfile,nextline,outfile,ModRep)
            print(nextline)
    print(OUTFN + ": " + str(ModRep) + " modules replaced")
    rawfile.close()
    outfile.close()

