#!/bin/bash
# TESTXYLEM -  (version 8 May 2023)
# Run XYLEM programs on test datasets.
# 
# 'textxylem.sh' with no arguments tests all xylem programs.
# 'testxylem program_name' tests the one program
# eg. 'testxylem.sh splitdb' tests splitdb

#####################################################################
# set default parameters
#####################################################################

# RM_CMD - command to be used for removing files and directories
if [ -e /usr/bin/rm ]
	then
   RM_CMD=/usr/bin/rm
else
   if [ -e /bin/rm ]
   	then
      RM_CMD=/bin/rm
   else
      RM_CMD=rm
   fi
fi

# Uncomment the correct BINDIR for your system
#BINDIR=../bin-solaris-sparc
#BINDIR=../bin-solaris-amd64
#BINDIR=../bin-linux-intel
BINDIR=../bin-linux-x86_64
#BINDIR=../bin-osx-x86_64
#BINDIR=../bin-macos-arm64

# set TESTSET to programs to be tested.
NUMARGS=$#
if [[ $NUMARGS == 0 ]]
then 
   TESTSET="splitdb getloc getob reform ribosome xylem_shuffle\
                  fetch features"
else
   TESTSET=$1
fi

#####################################################################
# test programs one at a time
#####################################################################
for PROGRAM in $TESTSET
    do

  echo "Testing $PROGRAM ..."
  case $PROGRAM in
    ######### splitdb #######
    splitdb)
       # run the program
       $BINDIR/splitdb sample.gen temp.ano temp.wrp temp.ind

       # if successful, compare sample files with new output
       SUCCESS=$?
       if [[ $SUCCESS == 0 ]]
       	then
          echo "     splitdb successful"
          echo "     Comparing sample files with new output (temp.*)..."
          diff sample.ano temp.ano >result
          if [ ! -s "${result}" ]
          	then
             echo "     .ano files compare okay"
             $RM_CMD temp.ano result
          else
             echo ">>>> sample.ano is different from temp.ano"
             echo ">>>> see difference.ano"
             mv result difference.ano
          fi 
          diff sample.wrp temp.wrp >result
          if [ ! -s "${result}" ]
          	then
             echo "     .wrp files compare okay"
             $RM_CMD temp.wrp result
          else
             echo ">>>> sample.wrp is different from temp.ano"
             echo ">>>> see difference.wrp"
             mv result difference.wrp
          fi
          diff sample.ind temp.ind >result
          if [ ! -s "${result}" ]
          	then
             echo "     .ind files compare okay"
             $RM_CMD temp.ind result
          else
             echo ">>>> sample.ind is different from temp.ind"
             echo ">>>> see difference.ind"
             mv result difference.ind
          fi 
        else
          echo ">>>> splitdb failed"
       fi
      ;;

    ######### getloc #######
    getloc)
       # run the program
       $BINDIR/getloc sample.nam sample.ano sample.wrp sample.ind temp.gen 

       # if successful, compare sample files with new output
       SUCCESS=$?
       if [[ $SUCCESS == 0 ]]
       	then
          echo "     getloc successful"
          echo "     Comparing sample files with new output (temp.*)..."
          diff sample.gen temp.gen >result
          if [ ! -s "${result}" ]
          	then
             echo "     .gen files compare okay"
             $RM_CMD temp.gen result
          else
             echo ">>>> sample.gen is different from temp.gen"
             echo ">>>> see difference.gen"
             mv result difference.gen
          fi 
        else
          echo ">>>> getloc failed"
       fi
      ;;


    ######### getob #######
    getob)
       # run the program
       $BINDIR/getob sample.inp sample.nam sample.ano sample.wrp sample.ind\
                     temp.getob.msg temp.getob.out temp.getob.exp

       # if successful, compare sample files with new output
       SUCCESS=$?
       if [[ $SUCCESS == 0 ]]
       	then
          echo "     getob successful"
          echo "     Comparing sample files with new output (temp.*)..."
          diff sample.msg temp.getob.msg >result
          if [ ! -s "${result}" ]
          	then
             echo "     .msg files compare okay"
             $RM_CMD temp.getob.msg result
          else
             echo ">>>> sample.msg is different from temp.getob.msg"
             echo ">>>> see difference.getob.msg"
             mv result difference.getob.msg
          fi 
          diff sample.out temp.getob.out >result
          if [ ! -s "${result}" ]
          	then
             echo "     .out files compare okay"
             $RM_CMD temp.getob.out result
          else
             echo ">>>> sample.out is different from temp.getob.out"
             echo ">>>> see difference.getob.out"
             mv result difference.getob.out
          fi 
          diff sample.exp temp.getob.exp >result
          if [ ! -s "${result}" ]
          	then
             echo "     .exp files compare okay"
             $RM_CMD temp.getob.exp result
          else
             echo ">>>> sample.exp is different from temp.getob.exp"
             echo ">>>> see difference.getob.exp"
             mv result difference.getob.exp
          fi 
        else
          echo ">>>> getob failed"
       fi
      ;;

    ######### reform #######
    reform)
       # run the program
       $BINDIR/reform -g -p -c -ff <sample.aln > temp.ref

       # if successful, compare sample files with new output
       SUCCESS=$?
       if [[ $SUCCESS == 0 ]]
       	then
          echo "     reform successful"
          echo "     Comparing sample files with new output (temp.*)..."
          diff sample.ref temp.ref >result
          if [ ! -s "${result}" ]
          	then
             echo "     .ref files compare okay"
             $RM_CMD temp.ref result
          else
             echo ">>>> sample.ref is different from temp.ref"
             echo ">>>> see difference.ref"
             mv result difference.ref
          fi 
        else
          echo ">>>> reform failed"
       fi
      ;;

    ######### ribosome #######
    ribosome)
       # run the program
       $BINDIR/ribosome <sample.out > temp.pep

       # if successful, compare sample files with new output
       SUCCESS=$?
       if [[ $SUCCESS == 0 ]]
       	then
          echo "     ribosome successful"
          echo "     Comparing sample files with new output (temp.*)..."
          diff sample.pep temp.pep >result
          if [ ! -s "${result}" ]
          	then
             echo "     .pep files compare okay"
             $RM_CMD temp.pep result
          else
             echo ">>>> sample.pep is different from temp.pep"
             echo ">>>> see difference.pep"
             mv result difference.pep
          fi 
        else
          echo ">>>> ribosome failed"
       fi
      ;;

    ######### prot2nuc #######
    prot2nuc)
       # run the program
       $BINDIR/prot2nuc <sample.pro > temp.prot2nuc

       # if successful, compare sample files with new output
       SUCCESS=$?
       if [[ $SUCCESS == 0 ]]
       	then
          echo "     prot2nuc successful"
          echo "     Comparing sample files with new output (temp.*)..."
          diff sample.prot2nuc temp.prot2nuc >result
          if [ ! -s "${result}" ]
          	then
             echo "     .prot2nuc files compare okay"
             #$RM_CMD temp.prot2nuc result
          else
             echo ">>>> sample.prot2nuc is different from temp.prot2nuc"
             echo ">>>> see difference.prot2nuc"
             mv result difference.prot2nuc
          fi 
        else
          echo ">>>> prot2nuc failed"
       fi
      ;;


   ######### xylem_shuffle #######
    xylem_shuffle)
       # run the program
       $BINDIR/xylem_shuffle <sample.wrp > temp.xshuf

       SUCCESS=$?
       if [[ $SUCCESS == 0 ]]
       	then
          echo "     xylem_shuffle successful"
          $RM_CMD temp.shuf
        else
          echo ">>>> xylem_shuffle failed"
       fi
      ;;

    ######### fetch #######
    fetch)
       # run the program
       $BINDIR/fetch -G sample sample.nam temp.gen

       # if successful, compare sample files with new output
       SUCCESS=$?
       if [[ $SUCCESS == 0 ]]
       	then
          echo "     fetch successful"
          echo "     Comparing sample files with new output (temp.*)..."
          diff sample.gen temp.gen >result
          if [ ! -s "${result}" ]
          	then
             echo "     .gen files compare okay"
             $RM_CMD temp.gen result
          else
             echo ">>>> sample.gen is different from temp.gen"
             echo ">>>> see difference.gen"
             mv result difference.gen
          fi 
        else
          echo ">>>> fetch failed"
       fi
      ;;

    ######### features #######
    features)
       # run the program
       cp sample.exp temp.exp
       $BINDIR/features -u sample -E temp.exp 

       # if successful, compare sample files with new output
       SUCCESS=$?
       if [[ $SUCCESS == 0 ]]
       	then
          echo "     features successful"
          echo "     Comparing sample files with new output (temp.*)..."
          diff sample.out temp.out >result
          if [ ! -s "${result}" ]
          	then
             echo "     .out files compare okay"
             $RM_CMD temp.out result
          else
             echo ">>>> sample.out is different from temp.out"
             echo ">>>> see difference.out"
             mv result difference.out
          fi 
        else
          echo ">>>> features failed"
       fi
       $RM_CMD temp.msg temp.exp
      ;;

    esac

done #foreach

echo TEST OF XYLEM COMPLETE
